@@ -1,10 +1,10 @@
 ##----------------------------------------------------------------------------
 ## A real Try Catch Block Implementation Using Perl Filter - ~/lib/Nice/Try.pm
-## Version v1.3.1
+## Version v1.3.2
 ## Copyright(c) 2022 DEGUEST Pte. Ltd.
 ## Author: Jacques Deguest <jack@deguest.jp>
 ## Created 2020/05/17
-## Modified 2022/07/28
+## Modified 2023/01/13
 ## All rights reserved
 ## 
 ## This program is free software; you can redistribute  it  and/or  modify  it
@@ -28,7 +28,7 @@ BEGIN
     use Scalar::Util ();
     use List::Util ();
     use Want ();
-    our $VERSION = 'v1.3.1';
+    our $VERSION = 'v1.3.2';
     our $ERROR;
     our( $CATCH, $DIED, $EXCEPTION, $FINALLY, $HAS_CATCH, @RETVAL, $SENTINEL, $TRY, $WANTARRAY );
 }
@@ -1207,13 +1207,13 @@ EOT
         }
         
         my $last_return_block = <<EOT;
-if( CORE::defined( \$Nice::Try::WANTARRAY ) and 
+if( ( CORE::defined( \$Nice::Try::WANTARRAY ) || ( defined( \$Nice::Try::BREAK ) && \$Nice::Try::BREAK eq 'return' ) ) and 
     (
       !Scalar::Util::blessed( \$Nice::Try::RETVAL[0] ) or 
       ( Scalar::Util::blessed( \$Nice::Try::RETVAL[0] ) && !\$Nice::Try::RETVAL[0]->isa( 'Nice::Try::SENTINEL' ) ) 
     ) ) 
 {
-    unless( CORE::defined( \$Nice::Try::BREAK ) )
+    if( !CORE::defined( \$Nice::Try::BREAK ) || \$Nice::Try::BREAK eq 'return' )
     {
         if( CORE::defined( \$Nice::Try::WANT ) && CORE::length( \$Nice::Try::WANT ) )
         {
@@ -1235,6 +1235,10 @@ if( CORE::defined( \$Nice::Try::WANTARRAY ) and
                 {
                     \$Nice::Try::BREAK = 'redo';
                 }
+                elsif( defined( \$Nice::Try::BREAK ) && \$Nice::Try::BREAK eq 'return' )
+                {
+                    CORE::return( \$Nice::Try::RETVAL[0] );
+                }
             }
             elsif( \$Nice::Try::WANT eq 'OBJECT' )
             {
@@ -1417,18 +1421,22 @@ sub _process_loop_breaks
             # $self->_browse( $e );
             # If we found a break word without a label, i.e. next, last, redo, 
             # we replace it with a special return statement
-            if( ( scalar( @$words ) == 1 ||
-                  ( scalar( @$words ) > 1 && $word2 =~ /^(for|foreach|given|if|unless|until|while)$/ )
+            if( (
+                  scalar( @$words ) == 1 ||
+                  ( scalar( @$words ) > 1 && $word2 =~ /^(for|foreach|given|if|unless|until|while)$/ ) ||
+                  $word1 eq 'return'
                 ) && 
                 (
                   $word1 eq 'next' ||
                   $word1 eq 'last' ||
-                  $word1 eq 'redo' ) )
+                  $word1 eq 'redo' ||
+                  $word1 eq 'return'
+                ) )
             {
                 # We add our special return value. Notice that we use 'return' and not 
                 # 'CORE::return'. See below why.
                 # my $break_code = qq{return( '__} . uc( $word1 ) . qq{__' )};
-                my $break_code = q{$Nice::Try::BREAK='} . $word1 . qq{', return};
+                my $break_code = q{$Nice::Try::BREAK='} . $word1 . ( $word1 eq 'return' ? "', $e" : qq{', return} );
                 # e.g. next if( $i == 2 );
                 # next and if are both treated as 'word' by PPI
                 if( scalar( @$words ) > 1 )
@@ -1448,8 +1456,8 @@ sub _process_loop_breaks
                 $new_elem->remove;
                 $self->_message( 5, "New element is object '", sub{ overload::StrVal( $new_elem ) }, "' -> $new_elem" ) if( $self->{debug} >= 5 );
                 # Not yet implemented as of 2021-05-11 dixit PPI, so we use a hack to make it available anyhow
-                $e->replace( $new_elem );
                 $self->_message( 5, "Updated element now is '$e' for class '", $e->class, "' and parent class '", $e->parent->class, "'." ) if( $self->{debug} >= 5 );
+                $e->replace( $new_elem );
                 # 2021-05-12 (Jacques): I have to do this workaround, because weirdly enough
                 # PPI (at least with PPI::Node version 1.270) will refuse to add our element
                 # if the 'return' word is 'CORE::return' so, we add it without and change it after
@@ -1828,7 +1836,7 @@ And you also have granular power in the catch block to filter which exception to
 
 =head1 VERSION
 
-    v1.3.1
+    v1.3.2
 
 =head1 DESCRIPTION
 
@@ -2294,7 +2302,7 @@ Since, try-catch block can be nested, the following would work too:
 
 =head1 EXCEPTION CLASS
 
-As mentioned above, you can use class when raising exceptions and you can filter them in a variety of way when you catch them.
+As mentioned above, you can use class when raising exceptions and you can filter them in a variety of ways when you catch them.
 
 Here are your options (replace C<Exception::Class> with your favorite exception class):
 
